/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2007  SEGGER Microcontroller Systeme GmbH         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.40c                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : LED.c
Purpose : LED interface for AT91SAM7Ae-EK
--------  END-OF-HEADER  ---------------------------------------------
*/

#define LED_C
#include "LED.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/

#define _AT91C_PIOA_BASE  (0xFFFFF400)
#define _AT91C_PIOB_BASE  (0xFFFFF600)

#define _AT91C_PMC_BASE   (0xFFFFFC00)

#define _PIOA_ID          (2)
#define _PIOB_ID          (3)
#define _PIO_PER_OFFS     (0x00)
#define _PIO_OER_OFFS     (0x10)
#define _PIO_SODR_OFFS    (0x30)    /* Set output data              */
#define _PIO_CODR_OFFS    (0x34)    /* Clear output data register   */
#define _PIO_ODSR_OFFS    (0x38)    /* Output data status register  */
#define _PIO_PDSR_OFFS    (0x3c)    /* Pin data status register     */

/****** SFRs and identifiers used for LED demo **********************/

#define _PMC           (*(volatile unsigned int*) _AT91C_PMC_BASE)

/****** Assign LEDs to Ports ****************************************/

/****** Target specific configuration *******************************/

#define _LEDPORT_BASE  (_AT91C_PIOA_BASE)
#define _LEDPORT_ID    (_PIOA_ID)
#define _LED0_BIT      (20) /* LED1 on SAM7A3-EK board active low */
#define _LED1_BIT      (21) /* LED2 on SAM7A3-EK board active low */

#define _LEDPORT_PER   (*(volatile unsigned int*) (_LEDPORT_BASE + _PIO_PER_OFFS))
#define _LEDPORT_OER   (*(volatile unsigned int*) (_LEDPORT_BASE + _PIO_OER_OFFS))
#define _LEDPORT_SODR  (*(volatile unsigned int*) (_LEDPORT_BASE + _PIO_SODR_OFFS))
#define _LEDPORT_CODR  (*(volatile unsigned int*) (_LEDPORT_BASE + _PIO_CODR_OFFS))
#define _LEDPORT_STATE (*(volatile unsigned int*) (_LEDPORT_BASE + _PIO_ODSR_OFFS))

#define _LEDMASK_ALL   ((1 << _LED0_BIT) | (1 << _LED1_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       LED_Init()
*/
void LED_Init(void) {
  _PMC |= (1 << _LEDPORT_ID);     /* Enable peripheral clock for LED-Port    */
  _LEDPORT_PER   = _LEDMASK_ALL;  /* Enable Port           */
  _LEDPORT_OER   = _LEDMASK_ALL;  /* Switch to output mode */
  _LEDPORT_SODR |= _LEDMASK_ALL;  /* Initially clear LEDs  */
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void LED_SetLED0(void) {
  _LEDPORT_CODR = (1 << _LED0_BIT);         /* Switch on LED 0  */
}

void LED_ClrLED0(void) {
  _LEDPORT_SODR = (1 << _LED0_BIT);         /* Switch off LED 0 */
}

void LED_ToggleLED0(void) {
  if (_LEDPORT_STATE & (1 << _LED0_BIT)) {  /* LED is switched off */
    LED_SetLED0();
  } else {
    LED_ClrLED0();
  }
}

void LED_SetLED1(void) {
  _LEDPORT_CODR = (1 << _LED1_BIT);         /* Switch on LED1  */
}

void LED_ClrLED1(void) {
  _LEDPORT_SODR = (1 << _LED1_BIT);         /* Switch off LED1 */
}

void LED_ToggleLED1(void) {
  if (_LEDPORT_STATE & (1 << _LED1_BIT)) {  /* LED is switched off */
    LED_SetLED1();
  } else {
    LED_ClrLED1();
  }
}

/****** End Of File *************************************************/


