;******************************************************************************
;
;    Copyright (C) 2005 Oki Electric Industry Co.,LTD.
;
;    System Name  :  ML69Q6203 CPU Board
;    Module Name  :  Startup routine
;    File   Name  :  init.s
;    Revision     :  01.00
;    Date         :  2005/03/06      First version
;                    2005/04/14      Second version
;
;******************************************************************************
        PRESERVE8

        INCLUDE define.s        ; common definitions

        IMPORT __use_no_semihosting_swi
        IMPORT __use_two_region_memory
        
        IF :DEF:COUNT_FIQ         
        IMPORT  count_interval  ; may be used to count FIQs and have to be defined elsewhere 
        ENDIF
        
;        IMPORT  IRQ_Stack_Top
;        IMPORT  FIQ_Stack_Top
;        IMPORT  SVC_Stack_Top
;        IMPORT  USR_Stack_Top

;;; Copyright ARM Ltd 2001. All rights reserved.

        AREA Vect, CODE, READONLY

; --- setup interrupt / exception vectors

        EXPORT  ENTRY_POINT
        IMPORT  OS_IRQ_SERVICE
        
        
ENTRY_POINT
                B       int_RESET       ; reset
int_UND         B       int_UND         ; undefined instruction
int_SWI         B       int_SWI         ; swi
int_PABT        B       int_PABT        ; prefetch abort
int_DABT        B       int_DABT        ; data abort
                NOP                     ; reserved vector
                B       OS_IRQ_SERVICE  ; B int_IRQ ; irq

; --- The FIQ handler starts here

        SUB     lr_FIQ, lr_FIQ, #4      ; construct the return address
        STMFD   sp_FIQ!, {r0-r3, lr_FIQ}; store low registers to FIQ stack

        IF :DEF:COUNT_FIQ               ; May be used to count FIQs
        LDR     r0,=count_interval      ; load variable address
        LDR     r1,[r0]
        ADD     r1, r1, #10             ; each FIQ casuses increment in count_interval
        STR     r1, [r0]
        ENDIF

        LDR     r0, =FIQ            ; load FIQ register address
Debounce_FIQ                        ; do not enable FIQ till FIQ source is de-asserted
        MOV     r3, #0
Debounce_FIQ100
        LDR     r1, [r0]            ; load contents of FIQ
        MOV     r2, #0x1
        AND     r1, r1, r2          ; zero out the high bits
        CMP     r1, #0x0            ; is FIQ still pending?
        BNE     Debounce_FIQ
        ADD     r3, r3,#1
        CMP     R3, #100
        BNE     Debounce_FIQ100

        LDMFD   sp_FIQ!, {r0-r3, pc}^  ; restore registers & return from FIQ
;;-------------------------------------------------------------------------------------------------

        EXPORT  Reset_Handler

Reset_Handler


;:*************************************************************************************************
;; Reset Handler                                                                                  *
;;*************************************************************************************************
int_RESET
        BL      setup_ex_sram_rom
        BL      setup_ex_dram
        BL      setup_protection_unit_cache

; --- initialize stack pointer registers


; Enter each mode in turn and set up the stack pointer
        IMPORT  top_of_stacks       ; defined in stack.s and located by scatter file
        LDR     r0, =top_of_stacks

        MSR     CPSR_c, #Mode_FIQ:OR:I_Bit:OR:F_Bit ; No interrupts
        SUB     sp, r0, #Offset_FIQ_Stack


        MSR     CPSR_c, #Mode_ABT:OR:I_Bit:OR:F_Bit ; No interrupts
        SUB     sp, r0, #Offset_ABT_Stack

        MSR     CPSR_c, #Mode_UND:OR:I_Bit:OR:F_Bit ; No interrupts
        SUB     sp, r0, #Offset_UND_Stack

        MSR     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit ; No interrupts
        SUB     sp, r0, #Offset_IRQ_Stack

        MSR     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit ; No interrupts        
        SUB     sp, r0, #Offset_SVC_Stack


; --- now enter the C code
        IMPORT  __main      
        B       __main             ; branch to main function with c libraries.

END_LOOP
        B       END_LOOP


    
;:*************************************************************************************************
;; Setup external SRAM/ROM                                                                        *
;;*************************************************************************************************
setup_ex_sram_rom
        LDR     R0, =BIC_BASE
        ;; setup BWC register(ROM:16bit,RAM:16bit)
        MOV     R1, #0x28
        STR     R1, [R0, #0x0]
        ;; setup ROMAC register()
        MOV     R1, #0x3
        STR     R1, [R0, #0x4]
        ;; setup RAMAC register()
        MOV     R1, #0x1
        STR     R1, [R0, #0x8]

        MOV     pc, lr

;:*************************************************************************************************
;; Setup external DRAM                                                                            *
;;*************************************************************************************************
setup_ex_dram
        LDR     R0, =DRAMC_BASE
        ;; waite 200usec or more
        MOV     R1, #0x1000
wait_loop
        SUBS    R1, R1, #1
        BPL     wait_loop
        ;; DRAM bus width control register (DBWC@0x7818_0000)
        LDR     R1, =0x2        ; 16bit
        STR     R1, [R0, #0]
        ;; DRAM control register (DRMC@0x7818_0004)
        LDR     R1, =0x02       ; column:10bit, type of DRAM:SDRAM, precharge latency:2clk
        STR     R1, [R0, #0x4]  ; auto shift to power down mode:disable, CBR refresh:disable
        ;; DRAM parameter control register (DRPC@0x7818_0008)
        LDR     R1, =0x9        ; slowest
        STR     R1, [R0, #0x8]
        ;; RFCG register(RFCG@0x7818_001C)
        LDR     R1, =0x3C       ; select refreqa(8usec)
        STR     R1, [R0, #0x1C]
        ;; DRAM refresh cycle control register(RFSH@0x7818_0014)
        LDR     R1, =0x1        ; refreqa(8usec) / 1
        STR     R1, [R0, #0x14]
        ;; DRAM power down mode control register (PDWC@0x7818_0018)
        LDR     R1, =0xF        ; 16cycle
        STR     R1, [R0, #0x18]
        ;; all bank pre-charge
        LDR     R1, =0x4        ; all bank precharge command
        STR     R1, [R0, #0x10]
        ;; CBR x 8
        LDR     R1, =0x5        ; CBR refresh command
        STR     R1, [R0, #0x10]
        STR     R1, [R0, #0x10]
        STR     R1, [R0, #0x10]
        STR     R1, [R0, #0x10]
        STR     R1, [R0, #0x10]
        STR     R1, [R0, #0x10]
        STR     R1, [R0, #0x10]
        STR     R1, [R0, #0x10]
        ;; SDRAM mode register (SDMD@0x7818_000C)
        LDR     R1, =0x80       ; CL2
        STR     R1, [R0, #0xC]
        ;; DRAM control register (DRMC@0x7818_0004)
        LDR     R1, =0x82       ; column:10bit, type of DRAM:SDRAM, precharge latency:2clk
        STR     R1, [R0, #0x4]  ; auto shift to power down mode:disable, CBR refresh:enable

        MOV     pc, lr

;:*************************************************************************************************
;; Setup protection unit                                                                          *
;;*************************************************************************************************
setup_protection_unit_cache
 
        ;; setup data cache (Write to CP15:Reg2)
        LDR     R0, =0x1E       ;  area1,2,3(b0001_1110) : enable 
        MCR     p15, 0, R0, c2, c0, 0
 
        ;; setup instruction cache (Write to CP15:Reg2)
        LDR     R0, =0x1A       ; area1,3(b0001_1010) : enable 
        MCR     p15, 0, R0, c2, c0, 1
        
        ;; setup write buffer (Write to CP15:Reg3)
        LDR     R0, =0x04       ; area2(b0001_0100) : enable      
        MCR     p15, 0, R0, c3, c0, 0

        ;; setup data access permission (Write to CP15:Reg5)
        LDR     R0, =0x00036363   ; area0,2,4 : RW/RW, area1,3 : RO/RO
        MCR     p15, 0, R0, c5, c0, 2; data access permission

        ;; setup instruction access permission
        LDR     R0, =0x00066666  
        MCR     p15, 0, R0, c5, c0, 3; instruction access permission

        ;; setup base size, address and enable/disable (Write to CP15:Reg6)
        LDR     R0, =0x3F       ; size : 4GB,   base address : 0x00000000
        MCR     p15, 0, R0, c6, c0, 0; area0
        LDR     R0, =0x35       ; size : 128MB, base address : 0x00000000
        MCR     p15, 0, R0, c6, c1, 0; area1
        LDR     R0, =0xC0000035 ; size : 128MB, base address : 0xC0000000
        MCR     p15, 0, R0, c6, c2, 0; area2
        LDR     R0, =0xC8000035 ; size : 128MB, base address : 0xC8000000
        MCR     p15, 0, R0, c6, c3, 0; area3
        LDR     R0, =0x50000035 ; size : 128MB, base address : 0x50000000
        MCR     p15, 0, R0, c6, c4, 0; area4

        ;; enable protection unit and cache(Write to CP15:Reg1)
        MRC     p15, 0, R0, c1, c0, 0; read control register
        ORR     R0, R0, #0x1000 ; instruction cache
        ORR     R0, R0, #0x5    ; data cache and protection unit
        MCR     p15, 0, R0, c1, c0, 0; write control register

        MOV     pc, lr


;*********************************************************************
;*
;*      Stack info for embOS
;*
;*********************************************************************

Len_FIQ_Stack    EQU     0
Len_IRQ_Stack    EQU     256
Len_ABT_Stack    EQU     0
Len_UND_Stack    EQU     0
Len_SVC_Stack    EQU     1024
Len_USR_Stack    EQU     0

Offset_FIQ_Stack         EQU     0
Offset_IRQ_Stack         EQU     Offset_FIQ_Stack + Len_FIQ_Stack
Offset_ABT_Stack         EQU     Offset_IRQ_Stack + Len_IRQ_Stack
Offset_UND_Stack         EQU     Offset_ABT_Stack + Len_ABT_Stack
Offset_SVC_Stack         EQU     Offset_UND_Stack + Len_UND_Stack
Offset_USR_Stack         EQU     Offset_SVC_Stack + Len_SVC_Stack
;

        EXPORT OS_GetStackInfo
OS_GetStackInfo
        IMPORT SVC_Stack_Base
        IMPORT SVC_Stack_Limit
        IMPORT IRQ_Stack_Base
        IMPORT IRQ_Stack_Limit

        IMPORT  top_of_stacks       ; defined in stack.s and located by scatter file
        LDR     r0, =top_of_stacks
; Get the IRQ Stack information
        LDR     r1, =IRQ_Stack_Limit
        sub     r2, r0, #Offset_IRQ_Stack
        STR     r2, [r1]
        LDR     r1, =IRQ_Stack_Base
        SUB     r2, r2, #Len_IRQ_Stack
        STR     r2, [r1]
; Get the Supervisor Stack information
        LDR     r1, =SVC_Stack_Limit
        sub     r2, r0, #Offset_SVC_Stack
        STR     r2, [r1]
        LDR     r1, =SVC_Stack_Base
        SUB     r2, r2, #Len_SVC_Stack
        STR     r2, [r1]
        BX      lr       

        END
